import os

dd_mm_yyyy = '%d-%m-%Y'
dd_mmm_yyyy = '%d-%b-%Y'
ddmmyyyy = '%d%m%Y'

equity_periods = ['1D', '1W', '1M', '3M', '6M', '1Y']

# ---------- column lists-----------------

price_volume_and_deliverable_position_data_columns = \
    ['Symbol', 'Series', 'Date', 'PrevClose', 'OpenPrice', 'HighPrice',
     'LowPrice', 'LastPrice', 'ClosePrice', 'AveragePrice', 'TotalTradedQuantity',
     'TurnoverInRs', 'No.ofTrades', 'DeliverableQty', '%DlyQttoTradedQty']

price_volume_data_columns = ['Symbol', 'Series', 'Date', 'PrevClose', 'OpenPrice', 'HighPrice',
                             'LowPrice', 'LastPrice', 'ClosePrice', 'AveragePrice',
                             'TotalTradedQuantity', 'Turnover', 'No.ofTrades']

deliverable_data_columns = ['Symbol', 'Series', 'Date', 'TradedQty', 'DeliverableQty', '%DlyQttoTradedQty']

bulk_deal_data_columns = ['Date', 'Symbol', 'SecurityName', 'ClientName', 'Buy/Sell', 'QuantityTraded',
                          'TradePrice/Wght.Avg.Price', 'Remarks']

block_deals_data_columns = ['Date', 'Symbol', 'SecurityName', 'ClientName', 'Buy/Sell', 'QuantityTraded',
                            'TradePrice/Wght.Avg.Price', 'Remarks']

short_selling_data_columns = ['Date', 'Symbol', 'SecurityName', 'Quantity']

bhavcopy_old = ['ISIN', 'TckrSymb', 'SctySrs', 'OpnPric', 'HghPric', 'LwPric', 'ClsPric', 'LastPric',
                'PrvsClsgPric', 'TtlTradgVol', 'TtlTrfVal', 'TradDt', 'TtlNbOfTxsExctd']

bhavcopy_new = ['ISIN', 'SYMBOL', 'SERIES', 'OPEN', 'HIGH', 'LOW', 'CLOSE', 'LAST', 'PREVCLOSE', 'TOTTRDQTY',
                'TOTTRDVAL', 'TIMESTAMP', 'TOTALTRADES']

future_price_volume_data_column = ['INSTRUMENT', 'SYMBOL', 'EXPIRY_DT', 'STRIKE_PRICE', 'OPTION_TYPE', 'MARKET_TYPE',
                                   'OPENING_PRICE', 'TRADE_HIGH_PRICE', 'TRADE_LOW_PRICE', 'CLOSING_PRICE',
                                   'LAST_TRADED_PRICE', 'PREV_CLS', 'SETTLE_PRICE', 'TOT_TRADED_QTY', 'TOT_TRADED_VAL',
                                   'OPEN_INT', 'CHANGE_IN_OI', 'MARKET_LOT', 'TIMESTAMP', 'UNDERLYING_VALUE']




import pandas as pd
import datetime as dt
import time
import zipfile
from io import BytesIO, StringIO
from nselib.libutil import *
from nselib.constants import *


def future_price_volume_data(symbol: str, instrument: str, from_date: str = None, to_date: str = None,
                             period: str = None):
    """
    get contract wise future price volume data set.
    :param instrument:  FUTIDX/FUTSTK
    :param symbol: symbol eg: 'SBIN' / 'BANKNIFTY'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '3M': last 3 month data
                            '6M': last 6 month data}
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)

    if instrument not in ['FUTIDX', 'FUTSTK']:
        raise ValueError(f'{instrument} is not a future instrument')

    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=future_price_volume_data_column)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 90:
            end_date = (from_date + dt.timedelta(90)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_future_price_volume_data(symbol=symbol, instrument=instrument,
                                               from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(91)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_future_price_volume_data(symbol: str, instrument: str, from_date: str, to_date: str):
    # print(from_date, to_date)
    data_df = pd.DataFrame()
    url = "https://www.nseindia.com/api/historical/foCPV?"
    payload = f"from={from_date}&to={to_date}&instrumentType={instrument}&symbol={symbol}&csv=true"
    try:
        data_dict = nse_urlfetch(url + payload).json()
    except Exception as e:
        raise ValueError(" Invalid parameters ")
    data_df = pd.DataFrame(data_dict['data']).drop(columns='TIMESTAMP')
    data_df.columns = cleaning_column_name(data_df.columns)
    return data_df[future_price_volume_data_column]


def option_price_volume_data(symbol: str, instrument: str, option_type: str = None, from_date: str = None,
                             to_date: str = None, period: str = None):
    """
    get contract wise option price volume data set. more than 90 days will take more time to collect data.
    :param option_type: PE/CE
    :param instrument:  OPTIDX/OPTSTK
    :param symbol: symbol eg: 'SBIN' / 'BANKNIFTY'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '3M': last 3 month data
                            '6M': last 6 month data}
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)

    if instrument not in ['OPTIDX', 'OPTSTK']:
        raise ValueError(f'{instrument} is not a future instrument')

    if option_type and option_type not in ['PE', 'CE']:
        raise ValueError(f'{option_type} is not a valid option type')

    option_type = [option_type] if option_type else ['PE', 'CE']
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=future_price_volume_data_column)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 90:
            end_date = (from_date + dt.timedelta(90)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        for opt_typ in option_type:
            data_df = get_option_price_volume_data(symbol=symbol, instrument=instrument, option_type=opt_typ,
                                                   from_date=start_date, to_date=end_date)
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
        from_date = from_date + dt.timedelta(91)
        load_days = (to_date - from_date).days

    return nse_df


def get_option_price_volume_data(symbol: str, instrument: str, option_type: str, from_date: str, to_date: str):
    data_df = pd.DataFrame()
    url = "https://www.nseindia.com/api/historical/foCPV?"
    payload = f"from={from_date}&to={to_date}&instrumentType={instrument}&symbol={symbol}" \
              f"&optionType={option_type}&csv=true"
    try:
        data_dict = nse_urlfetch(url + payload).json()
    except Exception as e:
        raise ValueError(" Invalid parameters ")
    data_df = pd.DataFrame(data_dict['data']).drop(columns='TIMESTAMP')
    data_df.columns = cleaning_column_name(data_df.columns)
    # print(data_df.columns)
    return data_df[future_price_volume_data_column]


def fno_bhav_copy(trade_date: str):
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = 'https://archives.nseindia.com/content/historical/DERIVATIVES/'
    payload = f"{str(trade_date.strftime('%Y'))}/{str(trade_date.strftime('%b').upper())}/" \
              f"fo{str(trade_date.strftime('%d%b%Y').upper())}bhav.csv.zip"
    request_bhav = nse_urlfetch(url + payload)
    bhav_df = pd.DataFrame()
    if request_bhav.status_code == 200:
        zip_bhav = zipfile.ZipFile(BytesIO(request_bhav.content), 'r')
        for file_name in zip_bhav.filelist:
            if file_name:
                bhav_df = pd.read_csv(zip_bhav.open(file_name))
    elif request_bhav.status_code == 403:
        raise FileNotFoundError(f' Data not found, change the date...')
    bhav_df = bhav_df[['INSTRUMENT', 'SYMBOL', 'EXPIRY_DT', 'STRIKE_PR', 'OPTION_TYP', 'OPEN', 'HIGH', 'LOW',
                       'CLOSE', 'SETTLE_PR', 'CONTRACTS', 'VAL_INLAKH', 'OPEN_INT', 'CHG_IN_OI', 'TIMESTAMP']]
    return bhav_df


def participant_wise_open_interest(trade_date: str):
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    raw_data = pd.DataFrame()
    url = f"https://archives.nseindia.com/content/nsccl/fao_participant_oi_{str(trade_date.strftime('%d%m%Y'))}.csv"
    # payload = f"{str(for_date.strftime('%d%m%Y'))}.csv"
    file_chk = requests.get(url, headers=header)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available for : {trade_date}")
    try:
        data_df = pd.read_csv(url, engine='python', sep=',', quotechar='"', on_bad_lines='skip', skiprows=1,
                              skipfooter=1)
    except:
        data_df = pd.read_csv(url, engine='c', sep=',', quotechar='"', on_bad_lines='skip', skiprows=1)
        data_df.drop(data_df.tail(1).index, inplace=True)
        data_df.columns = [name.replace('\t', '') for name in data_df.columns]
    return data_df


def participant_wise_trading_volume(trade_date: str):
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    raw_data = pd.DataFrame()
    url = f"https://archives.nseindia.com/content/nsccl/fao_participant_vol_{str(trade_date.strftime('%d%m%Y'))}.csv"
    # payload = f"{str(for_date.strftime('%d%m%Y'))}.csv"
    file_chk = requests.get(url, headers=header)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available for : {trade_date}")
    try:
        data_df = pd.read_csv(url, engine='python', sep=',', quotechar='"', on_bad_lines='skip', skiprows=1,
                              skipfooter=1)
    except Exception:
        data_df = pd.read_csv(url, engine='c', sep=',', quotechar='"', on_bad_lines='skip', skiprows=1)
        data_df.drop(data_df.tail(1).index, inplace=True)
        data_df.columns = [name.replace('\t', '') for name in data_df.columns]
    return data_df


if __name__ == '__main__':
    df = future_price_volume_data("BANKNIFTY", "FUTIDX", from_date='17-06-2023', to_date='19-06-2023', period='1D')
    # df = participant_wise_trading_volume(trade_date='03-04-2023')
    print(df)
    print(df[df['EXPIRY_DT']=='27-Jul-2023'])



from .capital_market_data import price_volume_and_deliverable_position_data, price_volume_data, \
    deliverable_position_data, bulk_deal_data, block_deals_data, short_selling_data, \
    bhav_copy_with_delivery, bhav_copy



import pandas as pd
import datetime as dt
import time
import zipfile
from io import BytesIO, StringIO
from nselib.libutil import *
from nselib.constants import *


def price_volume_and_deliverable_position_data(symbol:str, from_date:str = None, to_date:str = None, period:str = None):
    """
    get Security wise price volume & Deliverable position data set. use get_nse_symbols() to get all symbols
    :param symbol: symbol eg: 'SBIN'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,'1W': for last 7 days data,
                            '1M': from last month same date, '6M': last 6 month data, '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=price_volume_and_deliverable_position_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_price_volume_and_deliverable_position_data(symbol=symbol, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_price_volume_and_deliverable_position_data(symbol:str, from_date:str, to_date:str):
    # print(from_date, to_date)
    data_df = pd.DataFrame()
    url = "https://www.nseindia.com/api/historical/securityArchives?"
    payload = f"from={from_date}&to={to_date}&symbol={symbol}&dataType=priceVolumeDeliverable&series=ALL&csv=true"
    data_text = nse_urlfetch(url+payload).text
    data_text = data_text.replace('\x82','').replace('â¹', 'In Rs')
    with open('file.csv', 'w') as f:
        f.write(data_text)
    f.close()
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ','') for name in data_df.columns]
    return data_df


def price_volume_data(symbol:str, from_date:str = None, to_date:str = None, period:str = None):
    """
    get Security wise price volume data set.
    :param symbol: symbol eg: 'SBIN'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,'1W': for last 7 days data,
                            '1M': from last month same date, '6M': last 6 month data, '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=price_volume_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_price_volume_data(symbol=symbol, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_price_volume_data(symbol:str, from_date:str, to_date:str):
    # print(from_date, to_date)
    url = "https://www.nseindia.com/api/historical/securityArchives?"
    payload = f"from={from_date}&to={to_date}&symbol={symbol}&dataType=priceVolume&series=ALL&csv=true"
    data_text = nse_urlfetch(url+payload).text
    with open('file.csv', 'w') as f:
        f.write(data_text)
    f.close()
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ','') for name in data_df.columns]
    return data_df


def deliverable_position_data(symbol:str, from_date:str = None, to_date:str = None, period:str = None):
    """
    get Security wise deliverable position data set.
    :param symbol: symbol eg: 'SBIN'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=deliverable_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_price_volume_data(symbol=symbol, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_deliverable_position_data(symbol:str, from_date:str, to_date:str):
    # print(from_date, to_date)
    url = "https://www.nseindia.com/api/historical/securityArchives?"
    payload = f"from={from_date}&to={to_date}&symbol={symbol}&dataType=deliverable&series=ALL&csv=true"
    data_text = nse_urlfetch(url+payload).text
    # data_text = data_text.replace('\x82','').replace('â¹', 'In Rs')
    with open('file.csv', 'w') as f:
        f.write(data_text)
    f.close()
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ','') for name in data_df.columns]
    return data_df


def bulk_deal_data(from_date:str = None, to_date:str = None, period:str = None):
    """
    get bulk deal data set.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=bulk_deal_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_bulk_deal_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_bulk_deal_data(from_date:str, to_date:str):
    # print(from_date, to_date)
    url = "https://www.nseindia.com/api/historical/bulk-deals?"
    payload = f"from={from_date}&to={to_date}&csv=true"
    data_text = nse_urlfetch(url+payload).text
    # data_text = data_text.replace('\x82','').replace('â¹', 'In Rs')
    with open('file.csv', 'w') as f:
        f.write(data_text)
    f.close()
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ','') for name in data_df.columns]
    return data_df


def block_deals_data(from_date:str = None, to_date:str = None, period:str = None):
    """
    get block deals data set.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=block_deals_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_block_deals_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_block_deals_data(from_date:str, to_date:str):
    # print(from_date, to_date)
    url = "https://www.nseindia.com/api/historical/block-deals?"
    payload = f"from={from_date}&to={to_date}&csv=true"
    data_text = nse_urlfetch(url+payload).text
    # data_text = data_text.replace('\x82','').replace('â¹', 'In Rs')
    with open('file.csv', 'w') as f:
        f.write(data_text)
    f.close()
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ','') for name in data_df.columns]
    return data_df


def short_selling_data(from_date:str = None, to_date:str = None, period:str = None):
    """
    get short selling data set.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=short_selling_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_short_selling_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def get_short_selling_data(from_date:str, to_date:str):
    # print(from_date, to_date)
    url = "https://www.nseindia.com/api/historical/short-selling?"
    payload = f"from={from_date}&to={to_date}&csv=true"
    data_text = nse_urlfetch(url+payload).text
    # data_text = data_text.replace('\x82','').replace('â¹', 'In Rs')
    with open('file.csv', 'w') as f:
        f.write(data_text)
    f.close()
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ','') for name in data_df.columns]
    return data_df


def bhav_copy_with_delivery(trade_date:str):
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = 'https://archives.nseindia.com/products/content/sec_bhavdata_full_'
    payload = f'{trade_date}.csv'
    request_bhav = nse_urlfetch(url)
    bhav_df = pd.DataFrame()
    if request_bhav.status_code == 200:
        bhav_df = pd.read_csv(StringIO(request_bhav.text), sep=', ', engine='python')
    elif request_bhav.status_code == 403:
        raise FileNotFoundError(f' Data not found, change the date...')
    return bhav_df[['SYMBOL', 'SERIES', 'OPEN_PRICE', 'HIGH_PRICE', 'LOW_PRICE', 'CLOSE_PRICE',
                                   'PREV_CLOSE', 'TTL_TRD_QNTY', 'TURNOVER_LACS', 'NO_OF_TRADES', 'DELIV_QTY',
                                   'DELIV_PER', 'DATE1']]


def bhav_copy(trade_date:str):
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = 'https://archives.nseindia.com/content/historical/EQUITIES/'
    payload = f"{str(trade_date.strftime('%Y'))}/{str(trade_date.strftime('%b').upper())}/" \
              f"cm{str(trade_date.strftime('%d%b%Y').upper())}bhav.csv.zip"
    request_bhav = nse_urlfetch(url+payload)
    bhav_df = pd.DataFrame()
    if request_bhav.status_code == 200:
        zip_bhav = zipfile.ZipFile(BytesIO(request_bhav.content), 'r')
        for file_name in zip_bhav.filelist:
            if file_name:
                bhav_df = pd.read_csv(zip_bhav.open(file_name))
    elif request_bhav.status_code == 403:
        raise FileNotFoundError(f' Data not found, change the date...')
    bhav_df = bhav_df[['SYMBOL', 'SERIES', 'OPEN', 'HIGH', 'LOW', 'CLOSE', 'LAST', 'PREVCLOSE', 'TOTTRDQTY',
                       'TOTTRDVAL', 'TIMESTAMP', 'TOTALTRADES']]
    return bhav_df


if __name__ == '__main__':
    df = bhav_copy(trade_date='01-06-2022')
    print(df)
    print(df.columns)




import os
from datetime import datetime, timedelta, date
import requests
import numpy as np
from dateutil.relativedelta import relativedelta
import pandas as pd
import threading
import six
import sys
from nselib.constants import *
from nselib.logger import mylogger

header = {
    "Connection": "keep-alive",
    "Cache-Control": "max-age=0",
    "DNT": "1",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                  "Chrome/111.0.0.0 Safari/537.36",
    "Sec-Fetch-User": "?1", "Accept": "*/*", "Sec-Fetch-Site": "none", "Sec-Fetch-Mode": "navigate",
    "Accept-Encoding": "gzip, deflate, br", "Accept-Language": "en-US,en;q=0.9,hi;q=0.8"
    }


def validate_date_param(from_date:str, to_date:str, period:str):
    if not period and (not from_date or not to_date):
        raise ValueError(' Please provide the valid parameters')
    elif period and period.upper() not in equity_periods:
        raise ValueError(f'period = {period} is not a valid value')

    try:
        from_date = datetime.strptime(from_date, dd_mm_yyyy)
        to_date = datetime.strptime(to_date, dd_mm_yyyy)
    except Exception as e:
        print(e)
        raise ValueError(f'either or both from_date = {from_date} || to_date = {to_date} are not valid value')

    time_delta = (to_date-from_date).days
    if time_delta < 1:
        raise ValueError(f'to_date should greater than from_date ')


def derive_from_and_to_date(from_date:str = None, to_date:str = None, period:str = None):
    if not period:
        return from_date, to_date
    today = date.today()
    conditions = [period.upper()=='1D',
                  period.upper()=='1W',
                  period.upper()=='1M',
                  period.upper()=='6M',
                  period.upper()=='1Y'
                  ]
    value = [today - timedelta(days=1),
             today - timedelta(weeks=1),
             today - relativedelta(months=1),
             today - relativedelta(months=6),
             today - relativedelta(months=12)]

    f_date = np.select(conditions,value, default=(today - timedelta(days=1)))
    from_date = pd.to_datetime(str(f_date)).strftime(dd_mm_yyyy)
    today = today.strftime(dd_mm_yyyy)
    return from_date, today


def cleaning_column_name(col:list):
    unwanted_str_list = ['FH_']
    new_col=col
    for unwanted in unwanted_str_list:
        new_col = [name.replace(f'{unwanted}', '') for name in new_col]
    return new_col


def nse_urlfetch(url):
    r_session = requests.session()
    nse_live = r_session.get("http://nseindia.com", headers=header)
    return r_session.get(url, headers=header)


def get_nselib_path():
    """
    Extract isap file path
    """
    mydir = os.getcwd()
    return mydir.split(r'\nselib', 1)[0]




if __name__ == '__main__':
    # data = derive_from_and_to_date('6M')
    print(derive_from_and_to_date('6M'))
